// Persistence of Vision Ray Tracer Scene Description File
// File: ?.pov
// Vers: 3.5
// Desc: Basic Scene Example
// Date: mm/dd/yy
// Auth: ?
// 
// +RF"moss.rad" +RFI +c

#version 3.7;

#include "colors.inc"    
#include "functions.inc"      
#include "newMoss.inc" 
#include "shapes.inc"   


#declare useTestTex=false;
#declare useFunctions=1;  
#declare showMoss=1;
#declare useMossHair=true;  
#declare useRad=0; // 1=load 2=save
#declare radQuality=3; // 1-10 1=low 10= high
#declare showRock=1;
#declare useFocalBlur=0;    
#declare isoAcc=0.0005; // 0.0005    
#declare isoGrad=1.4; // 2.19



global_settings {   
assumed_gamma 1.0
#if(useRad)
ambient_light 0
  radiosity {

#if(useRad = 2)
    pretrace_start 0.08           // start pretrace at this size
    pretrace_end   0.001           // end pretrace at this size
#end
#if(useRad = 1)
    pretrace_start 1           // start pretrace at this size
    pretrace_end   1           // end pretrace at this size
    always_sample off  // 
#end

    count 5*radQuality 15*radQuality                      // higher -> higher quality (1..1600) [35]

    nearest_count 5+(radQuality) 5+(radQuality/2)               // higher -> higher quality (1..10) [5]

    low_error_factor .5           // reduce error_bound during last pretrace step
    gray_threshold 0.0            // increase for weakening colors (0..1) [0]
    minimum_reuse 0.015           // reuse of old radiosity samples [0.015]
    adc_bailout 0.01/2
    error_bound 1.2-(radQuality/10)               // higher -> smoother, less accurate [1.8]
    recursion_limit 2             // how much interreflections are calculated (1..5+) [3]
    brightness 1                  // brightness of radiosity effects (0..1) [1]
    normal on                  // take surface normals into account [off]

  }
#end 



}

sphere  { 0 1 hollow
  pigment {
    gradient y
    color_map {
      [0.0 rgb <0.6,0.7,1.0>]
      [0.7 rgb <0.0,0.1,0.8>]
    }
  } 
  finish { ambient 1 diffuse 1 emission 0.5 }
  scale 10000
}

light_source {
  <0, 0, 0>            // light's position (translated below)
  color rgb (White+Gold)*1.5  // light's color
  area_light
  <50, 0, 0> <0, 0, 50> // lights spread out across this distance (x * z)
  4, 4                // total number of lights in grid (4x*4z = 16 lights)
  adaptive 0          // 0,1,2,3...
  jitter              // adds random softening of light
  circular            // make the shape of the light circular
  orient              // orient light
  translate <500, 150, -400>
  fade_distance 750
  fade_power 2

}  

// ----------------------------------------

camera {
  location  <0, 2.2, -3.5>
  direction 1.5*z
  right     x*image_width/image_height
  look_at   <0, 0.25,  0.25>
#if(useFocalBlur)
  aperture 0.05          // [0...N] larger is narrower depth of field (blurrier)
  blur_samples useFocalBlur*10       // number of rays per pixel for sampling
  focal_point <0.075, 1.825, -2.75>
  confidence 0.9+(0.09*useFocalBlur) //0.8999+(useFocalBlur/1000)           // [0...<1] when to move on while sampling (smaller is less accurate)
  variance 1/(4000*useFocalBlur)            // [0...1] how precise to calculate (smaller is more accurate)
#end

}  

// Moss textures       

#declare earthTex= material {
texture  
{ 
        pigment 
        { 
                granite 
                turbulence 0.3 
                omega 0.7 
                lambda 2.8
                colour_map
                {
                        [ 0.0 rgb <0.15, 0.07, 0.04>*0.01 ]
                        [ 0.5 rgb <0.22, 0.11, 0.05>*0.01 ]
                        [ 1.0 rgb <0.32, 0.14, 0.05>*0.01 ]
                }                                     
        }                         
        finish { ambient 0 specular 0.01 roughness 0.1 diffuse albedo 0.5 brilliance 5 } 
        normal
        {    
            average normal_map {
                [ 1.0 wrinkles 0.725 turbulence 0.325 omega 0.7 scale 0.012 ]
                [ 1.0 granite 1.25 turbulence 0.325 omega 0.7 scale 0.0135 ]
                [ 1.0 granite 0.8 turbulence 0.23 scale 0.015 ]
                }
        }
}     
interior { ior 1.6 }
}

#declare t1 = texture 
{ 
        pigment
        {
                wrinkles      
                omega 0.675
                lambda 2.5 
                turbulence 0.375
                colour_map
                {
                       [ 0.05 rgbf <0.15, 0.225, 0.025, 0.3> ]
                       [ 0.1 rgbf <0.03, 0.02, 0.0025, 0.2> ]
                       [ 0.25 rgbf <0.03, 0.02, 0.0025, 0.1> ]
                       [ 0.3 rgbf <0.15, 0.225, 0.025, 0.35> ]
                       [ 0.7 rgbf <0.185, 0.275, 0.025, 0.35> ]
                       [ 0.85 rgbf <0.2, 0.3, 0.035, 0.4> ]                                            
                       [ 1.0 rgbf <0.05, 0.02, 0.0035, 0.1> ]                                            
                }          
                scale 0.065
                 
                
        }     
        normal { granite 0.07 scale 0.005 }
        finish { ambient 0.0 specular 0.25 roughness 0.05 diffuse 0.5 0.2 brilliance 1.2 reflection { 0.01 0.005 fresnel on } conserve_energy  }
}          

#declare t2 = texture {
        pigment
        {
                wrinkles      
                omega 0.675
                lambda 2.5 
                turbulence 0.375
                colour_map
                {
                       [ 0.0 rgbf <0.05, 0.02, 0.0025, 0.1> ]
                       [ 0.7 rgbf <0.03, 0.02, 0.0025, 0.1> ]
                       [ 1.0 rgbf <0.03, 0.01, 0.0035, 0.1> ]                                            
                } 
        }     
        finish { ambient 0.0 specular 0.04 roughness 0.01 diffuse 1 brilliance 0.8 reflection { 0.01 0.005 fresnel on } conserve_energy  }
}

#declare hairTex = material {
texture 
{ 
    gradient y
    turbulence 0.31
    texture_map {
        [ 0.0 t1 ]
        [ 0.8 t1 ]
        [ 1.0 t2 ]        
    }             
    scale <.5, 0.5, .5>  
    translate <0, -0.3, 0> 
}   
// describes inside of a shape (similar to texture, but for inside, not surface)
  interior {
    ior 1.6              // index of refraction
    fade_power 1001      // values larger than 1000 give realistic exponential attenuation
    fade_distance 0.0001       // distance where light reaches half intensity
    fade_color <.1, .5, .1> // color for fading
  }

}

// Rock texture


#declare rockTexBase = texture {
        pigment { rgbt <0.025, 0.0175, 0.005, 0.01> }
        finish { specular 0.6 roughness 0.0075 diffuse 0.3 brilliance 5 reflection { 0.025 0.01 fresnel on metallic 0.4 } conserve_energy }
} 
texture {
        pigment {   bozo  
                    omega 0.4
                    lambda 2.0
                    turbulence 0.25
                colour_map {
                        [ 0.0 rgbt <0.5, 0.42, 0.3, 0> ]
                        [ 0.3 rgbt <0.4, 0.33, 0.25, 0> ]
                        [ 0.80 rgbt <0.5, 0.42, 0.33, 0.1> ]
                        [ 0.82 rgbt <1, 1, 1, 1> ]
                        [ 1.0 rgbt <1, 1, 1, 1> ]
                        
                }
                scale 0.025
        }

        finish { specular 0.01 roughness 0.2 brilliance 1.2 diffuse 0.7 }    
}  
texture { 
        pigment { wrinkles
        triangle_wave    
        omega 0.8
        lambda 3
        turbulence 1
                colour_map {
                        [ 0.0 rgbt <0.01, 0.0075, 0.0, 0.1> ]
                        [ 0.2 rgbt <0.01, 0.0075, 0.0, 0.8> ]
                        [ 0.4 rgbt <0.02, 0.01, 0.001, 0.1> ]
                        [ 0.7 rgbt <0.05, 0.035, 0.021, 0.7> ]
                        [ 0.98 rgbt <0.1, 0.075, 0.01, 1> ]
                        [ 1.0 rgbt <1, 1, 1,1.0> ]                       
                }                                 
                scale 0.5
        }
        finish { specular 0.01 roughness 0.2 brilliance 1.2 diffuse 0.5 }    
        normal { granite 0.5 scale 0.05 }
}


#declare rockTexLDirt = texture { rockTexBase }
texture { 
        pigment { wrinkles
        triangle_wave    
        omega 0.8
        lambda 3
        turbulence 1
                colour_map {
                        [ 0.0 rgbt <0.1, 0.05, 0.0, 0.2> ]
                        [ 0.2 rgbt <0.1, 0.05, 0.0, 0.8> ]
                        [ 0.4 rgbt <0.2, 0.125, 0.025, 0.2> ]
                        [ 0.8 rgbt <0.2, 0.125, 0.025, 1> ]
                        [ 0.8 rgbt <1, 1, 1,1.0> ]                       
                        [ 1.0 rgbt <1, 1, 1,1.0> ]                       
                }                                 
                scale 0.5
                rotate <20, 15, 10>
        }
        finish { brilliance 1.2 diffuse 0.45 }    
        normal { granite 0.5 scale 0.05 }
}          

#declare rockTexDirt = texture { rockTexBase }
texture { 
        pigment { wrinkles
        triangle_wave    
        omega 0.8
        lambda 3
        turbulence 1
                colour_map {
                        [ 0.0 rgbt <0.1, 0.035, 0.0, 0.2> ]
                        [ 0.2 rgbt <0.1, 0.035, 0.0, 0.6> ]
                        [ 0.75 rgbt <0.175, 0.1, 0.0125, 0.2> ]
                        [ 0.99 rgbt <0.175, 0.1, 0.0125, 1> ]
                        [ 0.99 rgbt <1, 1, 1,1.0> ]                       
                        [ 1.0 rgbt <1, 1, 1,1.0> ]                       
                }                                 
                scale 0.5
                rotate <20, 15, 10>
        }
        finish { brilliance 1.2 diffuse 0.45 }    
        normal { granite 0.5 scale 0.05 }
}

#declare rockTexVDirt = texture { rockTexBase }
texture { 
        pigment { wrinkles
        triangle_wave    
        omega 0.8
        lambda 3
        turbulence 1
                colour_map {
                        [ 0.0 rgbt <0.075, 0.01, 0.0, 0.1> ]
                        [ 0.2 rgbt <0.075, 0.01, 0.0, 0.3> ]
                        [ 0.75 rgbt <0.125, 0.0255, 0.00125, 0.1> ]
                        [ 1.0 rgbt <0.125, 0.0255, 0.00125, 0.5> ]
                }                                 
                scale 0.5
                rotate <20, 15, 10>
        }
        finish { brilliance 1.2 diffuse 0.45 }    
        normal { granite 0.5 scale 0.05 }
}

#declare rockTexSide = texture {
gradient z    
triangle_wave
turbulence 0.125
texture_map {
        [ 0.0  rockTexLDirt scale 0.25 ]
        [ 0.025  rockTexLDirt scale 0.25 ]
        [ 0.075  rockTexDirt scale 0.25 ]
        [ 0.1   rockTexVDirt scale 0.25 ]
        [ 0.3   rockTexVDirt scale 0.25 ]
        [ 0.4  rockTexLDirt scale 0.25 ]
        [ 1.0  rockTexLDirt scale 0.25 ]
        }  
        scale 2
        translate <0, 0, 0>
}

#declare rockTexTop = texture {
gradient y    
triangle_wave
turbulence 0.125
texture_map {
        [ 0.0  rockTexLDirt scale 0.25 ]
        [ 0.025  rockTexLDirt scale 0.25 ]
        [ 0.075  rockTexDirt scale 0.25 ]
        [ 0.1   rockTexVDirt scale 0.25 ]
        [ 1.0   rockTexVDirt scale 0.25 ]
        }  
        scale 2
        translate <0, -0.8, 0>
}


#declare rockTex = texture {  
gradient z    
triangle_wave
turbulence 0.05
texture_map {
        [ 0.0  rockTexSide ]
        [ 0.075  rockTexSide ]
        [ 0.1  rockTexTop ]
        [ 1.0 rockTexTop ]
        } 
        translate <0, 0, -1.1> 
}        
        
// ----------------------------------------

#declare rock=isosurface 
{
#if(useFunctions)
        function{f_rounded_box(x,y,z,0.01, 5, 3, 3)-(f_agate(x*1,y*1,z*1)*0.02)-(f_agate(x*10,y*10,z*10)*0.002)-(f_wrinkles(x*5,y*5,z*5)*0.025)-(f_wrinkles(x*0.5,y*0.5,z*0.5)*0.25)}
#else
        function{f_rounded_box(x,y,z,0.01, 5, 3, 3)}
#end
        threshold 0
        contained_by { box { <-6, -5, -5> <6, 5, 5> } }             
        accuracy isoAcc
        max_gradient isoGrad       

material {   
    texture { rockTex }
        interior { ior 1.6 } 
        }   
}    

union
{
#if(showRock)
        object { rock }
#end
#if(showMoss)
object { Place_Moss(rock, <-3.5, 10, -3.075>, <-3.4, 10, -3.0>, 0.94, <0, -1, 0>, 0.025, <1, 0.2, 1>, 25, 55, useMossHair, <0.05, 0.05, 0.05>, 4, MOSS_TYPE_1, hairTex, earthTex, 0.05) }
object { Place_Moss(rock, <-3.425, 10, -3.0>, <-3.3, 10, -2.8>, 0.94, <0, -1, 0>, 0.025, <1, 0.2, 1>, 25, 55, useMossHair, <0.05, 0.05, 0.05>, 4, MOSS_TYPE_1, hairTex, earthTex, 0.05) }
object { Place_Moss(rock, <-3.1, 10, -3.05>, <-3.0, 10, -2.7>, 0.94, <0, -1, 0>, 0.025, <1, 0.2, 1>, 25, 55, useMossHair, <0.05, 0.05, 0.05>, 4, MOSS_TYPE_1, hairTex, earthTex, 0.05) }
object { Place_Moss(rock, <-3.0, 10, -3.075>, <-2.75, 10, -2.96>, 0.94, <0, -1, 0>, 0.025, <1, 0.2, 1>, 25, 55, useMossHair, <0.05, 0.05, 0.05>, 4, MOSS_TYPE_1, hairTex, earthTex, 0.05) }
object { Place_Moss(rock, <-3.2, 2.85, -10>, <-3.1, 2.9, -10>, 0.94, <0, 0, 1>, 0.025, <1, 0.5, 0.1>, 25, 55, useMossHair, <0.05, 0.05, 0.05>, 4, MOSS_TYPE_1, hairTex, earthTex, 0.001) }
#end
        rotate <0, -46, 0>
        translate <0, -1.3, 1.5> 

}